// https://github.com/fil/array-blur v1.2.1 Copyright 2021 Fil
(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.d3 = global.d3 || {}));
}(this, (function (exports) { 'use strict';

const floor = Math.floor;

const max = Math.max;

const min = Math.min;

function arrayify(values) {
  return typeof values !== "object" || "length" in values ? values : Array.from(values);
}

function blurTransfer(V, r, n, vertical) {
  if (!r) return; // radius 0 is a noop

  const iterations = 3, r0 = Math.floor(r);
  // for a non-integer radius, interpolate between floor(r) and ceil(r)
  if (r === r0) {
    for (let i = 0; i < iterations; i++) {
      blurTransferInt(V, r, n, vertical);
    }
  } else {
    const frac = r - r0, frac_1 = 1 - frac;
    const data = V[0].slice();
    for (let i = 0; i < iterations; i++) {
      blurTransferInt(V, r0 + 1, n, vertical);
    }
    const data_ceil = V[0];
    V[0] = data;
    if (r0 > 0) {
      for (let i = 0; i < iterations; i++) {
        blurTransferInt(V, r0, n, vertical);
      }
    }
    for (let i = 0; i < data.length; i++) {
      V[0][i] = V[0][i] * frac_1 + data_ceil[i] * frac;
    }
  }
}

function blurTransferInt(V, r, n, vertical) {
  const [source, target] = V,
    m = floor(source.length / n),
    w = 2 * r + 1,
    w1 = 1 / w,
    ki = vertical ? m : 1,
    kj = vertical ? 1 : n,
    W = w * ki,
    R = r * ki;

  for (let j = 0; j < m; ++j) {
    const k0 = kj * j,
      kn = k0 + ki * (n - 1);
    for (let i = 0, sr = w * source[k0]; i < n + r; ++i) {
      const k = ki * i + kj * j;
      sr += source[min(k, kn)] - source[max(k - W, k0)];
      target[max(k - R, k0)] = sr * w1;
    }
  }
  V.reverse(); // target becomes V[0] and will be used as source in the next iteration
}

function blur() {
  let rx = 5,
    ry = rx,
    value,
    width;
  const V = [];

  function blur(data) {
    // reuse the V arrays if possible
    if (value || !V[0] || V[0].length !== data.length) {
      V[0] = value ? Float32Array.from(data, value) : Float32Array.from(data);
      V[1] = new Float32Array(V[0].length);
    } else {
      V[0].set(arrayify(data));
    }

    const n = width || V[0].length;
    const m = Math.round(V[0].length / n);

    blurTransfer(V, rx, n, false);
    blurTransfer(V, ry, m, true);

    V[0].width = n;
    V[0].height = m;
    return V[0];
  }

  blur.radius = _ =>  _ === undefined
    ? (rx + ry) / 2
    : (rx = ry = +_, blur);
  blur.radiusX = _ =>  _ === undefined
    ? rx : (rx = +_, blur);
  blur.radiusY = _ =>  _ === undefined
    ? ry : (ry = +_, blur);
  blur.width = _ =>
    _ === undefined ? width : (width = Math.round(+_), blur);
  blur.value = _ =>
    typeof _ === "function" ? (value = _, blur) : value;

  return blur;
}

exports.blur = blur;

Object.defineProperty(exports, '__esModule', { value: true });

})));
